/*****************************************************************************
*                                                                            *
* void      Dot_Diffusion(void)                                              *
*                                                                            *
******************************************************************************
*                                                                            *
* Erzeugt ein Binaerbild nach dem Dot-Diffusion-Verfahren. Dies ist eine     *
* Kombination von Ordered-Dither und Error-Diffusion.                        *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 24.08.1992                                               *
* Letzte Aenderung: 03.09.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input:                                                                     *
*                                                                            *
******************************************************************************
*                                                                            *
* Output:                                                                    *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: WriteText()                                                    *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void Dot_Diffusion(void)
{
  int i, j, KlassenNr, Gewicht;
  int x, y, PosX, PosY, NachbX, NachbY, Found, Error;
  int Grauwert;
  int SpaltenZahl, ZeilenZahl;
  long Index1, Offset;                      /* Adresse der einzelnen Bilder */

  int Wichtung[4][4];                       /* zur Fehlerverteilung         */

/* Eine weitere, moegliche Dot-Diffusion-Matrix
  static int Klasse[8][8]={{25,21,13,39,47,57,53,45},
                           {48,32,29,43,55,63,61,56},
                           {40,30,35,51,59,62,60,52},
                           {36,14,22,26,46,54,58,44},
                           {16, 6,10,18,38,42,50,24},
                           { 8, 0, 2, 7,15,31,34,20},
                           { 4, 1, 3,11,23,33,28,12},
                           {17, 9, 5,19,27,49,41,37}};
*/
  static int Klasse[8][8]={{34,48,40,32,29,15,23,31},
                           {42,58,56,53,21, 5, 7,10},
                           {50,62,61,45,13, 1, 2,18},
                           {38,46,54,37,25,17, 9,26},
                           {28,14,22,30,35,49,41,33},
                           {20, 4, 6,11,43,59,57,52},
                           {12, 0, 3,19,51,63,60,44},
                           {24,16, 8,27,39,47,55,36}};


  WriteText(1, 25, "Bitte etwas Geduld ...");

  /* Quell-Bild in Ziel-Bild kopieren, da dort direkt mit gearbeitet wird.  */
  for (Index1=0; Index1<PIC_SIZE; Index1++) Pictures[Index1+(long)PIC_SIZE]=Pictures[Index1];
  Index1=PIC_SIZE;

  Bild[ZIEL_PIC].Zeilen =Bild[QUELL_PIC].Zeilen;
  Bild[ZIEL_PIC].Spalten=Bild[QUELL_PIC].Spalten;

  /* Gewichtungsfaktoren berech. Mit wachsendem Abstand nimmt Gewichtung ab */
  for (i=0; i<3; i++)                                        /*   1  2  1   */
       for (j=0; j<3; j++)                                   /*   2  3  2   */
           Wichtung[i][j]=3-((i-1)*(i-1))-((j-1)*(j-1));     /*   1  2  1   */

  /* Dimensionen von Quell- und Ziel-Bild sind hier immer identisch         */
  SpaltenZahl=Bild[ZIEL_PIC].Spalten;
  ZeilenZahl =Bild[ZIEL_PIC].Zeilen;

  /* Mit der kleinsten Klasse wird gestartet. Zu finden ist der erste       */
  /* Punkt mit Klasse=KlassenNr.                                            */
  for (KlassenNr=0; KlassenNr<64; KlassenNr++) {
      y=0;
      Found=FALSE;                             /* aufhoeren sobald gefunden */
      while ((y<8) && (!Found)) {
         x=0;
         while ((x<8) && (!Found)) {
            if (Klasse[y][x]==KlassenNr)  Found=TRUE;
            else                          x++;
         }
         y++;
      }
      y--;            /* Korrektur, da die Zeile schon eins zuviel gezaehlt */

      /* Normierung der Gewichtskoeffizienten; dafuer betr. Nachbern ermitt.*/
      PosX=x+8;    /* Shift um ein Klasse[][]-Feld damit keine neg. Indizes */
      PosY=y+8;
      Gewicht=0;
      /* Umgebung des aktuellen Punktes nach groeeren Klassen durchsuchen  */
      for (NachbY=PosY-1; NachbY<=PosY+1; NachbY++)
          for (NachbX=PosX-1; NachbX<=PosX+1; NachbX++)
              if ( (Klasse[NachbY % 8][NachbX % 8]>KlassenNr) && ((NachbY!=PosY)||(NachbX!=PosX)) )
                 Gewicht += Wichtung[NachbY-PosY+1][NachbX-PosX+1];

      /* Nun mit Ordered-Dither starten. Dabei den gewichteten Fehler ber.  */
      /* Betreffende Klasse in jedem 8x8-Feld abarbeiten und Fehler verteil.*/
      for (j=y; j<ZeilenZahl; j+=8) {
          Offset=(long) j*SpaltenZahl;
          for (i=x; i<SpaltenZahl; i+=8) {
              Grauwert=Pictures[Index1+Offset+i];
              Error = Grauwert - (Pictures[Index1+Offset+(long)i]=(Grauwert>=GRAU) ? WEISS:SCHWARZ);
      
              /* Verteilen des Fehlers auf Nachbarn mit Klasse > KlassenNr  */
              for (NachbY=j-1; NachbY<=j+1; NachbY++)
                  for (NachbX=i-1; NachbX<=i+1; NachbX++)
                      if ( (NachbX>=0) && (NachbX<SpaltenZahl) && (NachbY>=0) && (NachbY<ZeilenZahl))
                           if (Klasse[NachbY % 8][NachbX % 8] > KlassenNr) {
                              Grauwert = Pictures[Index1+(long)NachbY*SpaltenZahl+(long)NachbX] + (Error*Wichtung[NachbY-j+1][NachbX-i+1])/Gewicht;
                              /* Clipping durchfuehren da hier keine        */
                              /* Int-Arrays zur Fehlerverteilung verwendet  */
                              if (Grauwert<SCHWARZ) Grauwert=SCHWARZ;
                              if (Grauwert>WEISS)   Grauwert=WEISS;
                              Pictures[Index1+(long)NachbY*SpaltenZahl+(long)NachbX] = Grauwert;
                           }
          }
      }
  }

}
