/*****************************************************************************
*                                                                            *
****  Sammlung von Routinen zur Graphik-Initialisierung und Bildausgabe   ****
*                                                                            *
******************************************************************************
*                                                                            *
* void      PlotPixel(long Spalte, long Zeile, unsigned char Farbe)          *
* void      Graph_Mode()                                                     *
* void      Text_Mode()                                                      *
* void      Set_LUT()                                                        *
* void      Make_Icon(int BildNr)                                            *
* void      Show_Icon(int BildNr)                                            *
* void      Show_Pic(int BildNr, int Position)                               *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


/*****************************************************************************
*                                                                            *
* void      PlotPixel(long Spalte, long Zeile, unsigned char Farbe)          *
*                                                                            *
******************************************************************************
*                                                                            *
* Setzt ein Bildpunkt an der Position (Spalte, Zeile) mit Farbe.             *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 19.08.1992                                               *
* Letzte Aenderung: 19.08.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: Spalte:   0 <= Spalte   <  S_RES_MAX    - Spalte auf dem Bildschirm *
*        Zeile:    0 <= Zeile    <  Z_RES_MAX    - Zeile auf dem Bildschirm  *
*        Farbe:    0 <= Farbe    <  MAX_COLOR    - Pixelfarbe (LUT-Eintrag)  *
*                                                                            *
*****************************************************************************/

void PlotPixel(long Spalte, long Zeile, unsigned char Farbe)
{
  unsigned int Index;              /* Index bei der linearen Adressierung   */

  Index = (unsigned int) (Zeile * 80) + (Spalte>>3);        /* Pixel-Index  */

  /* Abbilden der 256 Graustufen auf 15(16) Stueck. Index 1 ist mit Rot bel.*/
  Farbe = (Farbe>31) ? (Farbe>>4):( (Farbe>=CLIP_COLOR) ? 0:Farbe );

  /* Index 7 ist auf Weiss gesetzt fuer die Text-Ausgabe. Index 15 enthaelt */
  /* den Grauwert von Index 7. Daher Vertauschung notwendig.                */
  Farbe = (Farbe==7) ? 15:( (Farbe==15 ? 7:Farbe) );

  outport(0x3CE, 0);               /* Set/Reset-Register im Graphic ...     */
  outport(0x3CF, Farbe & 0x0F);    /* ... Controller mit Farbe laden.       */
  outport(0x3CE, 1);               /* Enable Set/Reset-Register und ...     */
  outport(0x3CF, 0xF);             /* ... alle vier Ebenen freigeben.       */

  outport(0x3CE, 8);               /* Bit-Mask-Register mit den zu ...      */
  outport(0x3CF, 128>>(Spalte & 0x07));  /* ... setzenden Bits laden.       */

  vgaram[Index]=vgaram[Index];     /* Dummy-Zugriff zum Pixel setzen.       */

  outport(0x3CF, 0xFF);            /* Bit-Mask-Register wieder freigeben    */
  outport(0x3CE, 1);               /* Enable Set/Reset-Register und ...     */
  outport(0x3CF, 0);               /* ... Ansprechen der Ebenen sperren.    */
}


/*****************************************************************************
*                                                                            *
* void      Graph_Mode()      -  12h                                         *
*                                                                            *
******************************************************************************
*                                                                            *
* Schaltet den Graphik-Modus GRAPH_MODE ein.                                 *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 19.08.1992                                               *
* Letzte Aenderung: 24.02.1993                                               *
*                                                                            *
*****************************************************************************/

void Graph_Mode(void)
{
  pc_cpu.h.ah = 0;                 /* Neuer Video-Modus                     */
                                   /* Graphik-0x12 = 640x480 bei 16 Farben  */
  pc_cpu.h.al = 0x12;              /* ueber Funktionsaufruf 00 setzen ...   */
  int86(0x10, &pc_cpu, &pc_cpu);   /* ... und durch Interrupt 10 aktivieren.*/

  vgaram = MK_FP(0xA000, 0);      /* Erzeugt einen Far-Pointer auf VGA-Segm.*/
}


/*****************************************************************************
*                                                                            *
* void      Text_Mode()                                                      *
*                                                                            *
******************************************************************************
*                                                                            *
* Schaltet den Text-Modus TEXT_MODE ein.                                     *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 19.08.1992                                               *
* Letzte Aenderung: 19.08.1992                                               *
*                                                                            *
*****************************************************************************/

void Text_Mode(void)
{
  pc_cpu.h.ah = 0;                 /* Text-Modus ueber den Funktionsaufruf  */
  pc_cpu.h.al = TEXT_MODE;         /* 00 setzen und ...                     */
  int86(0x10, &pc_cpu, &pc_cpu);   /* ... durch Interrupt 10 aktivieren.    */
}


/*****************************************************************************
*                                                                            *
* void      Set_LUT()                                                        *
*                                                                            *
******************************************************************************
*                                                                            *
* Initialisiert die Farbtabelle mit einem Graukeil und den Grundfarben       *
* Rot, Gruen und Blau an den Eintraegen [0], [1] und [2].                    *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 19.08.1992                                               *
* Letzte Aenderung: 19.08.1992                                               *
*                                                                            *
*****************************************************************************/

void Set_LUT(void)
{
  int i;
  rgb RGB_Tabelle[MAX_COLOR];      /* Farbtabelle mit MAX_COLOR Eintraegen  */
  unsigned char AC_Index[17];      /* Zuweisung DAC-Tabelle zu AC-Tabelle   */

  RGB_Tabelle[0].Rot   =   0;      /* Ueber Index [0] wird Scharz erzeugt   */
  RGB_Tabelle[0].Gruen =   0;
  RGB_Tabelle[0].Blau  =   0;
  RGB_Tabelle[1].Rot   =  63;      /* Ueber Index [1] wird Rot angesprochen */
  RGB_Tabelle[1].Gruen =   0;
  RGB_Tabelle[1].Blau  =   0;

  /* Farbtabelle von Eintrag 2 bis 15 als abgedunkelten Graukeil aufbauen   */
  for (i=2; i<16; i++) {
      RGB_Tabelle[i].Rot   = (unsigned char)(i*4-1);
      RGB_Tabelle[i].Gruen = (unsigned char)(i*4-1);
      RGB_Tabelle[i].Blau  = (unsigned char)(i*4-1);
  }
  /* Vertauschen von Eintrag Nr. 7 (==Grau) und Nr. 15 (==Weiss)            */
  RGB_Tabelle[15].Rot   =  RGB_Tabelle[7].Rot;
  RGB_Tabelle[15].Gruen =  RGB_Tabelle[7].Gruen;
  RGB_Tabelle[15].Blau  =  RGB_Tabelle[7].Blau;

  /* Es wird nicht die volle Intensitaet verwendet um das Fehlen der Stufe  */
  /* 1 (==Fast Schwarz) zu verbergen. Intensitaet 59 == Weiss               */
  RGB_Tabelle[7].Rot   =  (unsigned char)59;
  RGB_Tabelle[7].Gruen =  (unsigned char)59;
  RGB_Tabelle[7].Blau  =  (unsigned char)59;

  pc_cpu.h.ah=0x10;                /* Aktiviere Modus zum Setzen mehrerer...*/
  pc_cpu.h.al=0x12;                /* ... Farb-Palettenregister (LUT)       */
  pc_cpu.x.bx=0;                   /* Beginne bei Register Nummer 0 und ... */
  pc_cpu.x.cx=16;                  /* ... aendere die 16 Register           */
  s_cpu.es   =FP_SEG(RGB_Tabelle); /* Segmentadresse der LUT-Tabelle        */
  pc_cpu.x.dx=FP_OFF(RGB_Tabelle); /* Offsetadresse in diesem Segment und...*/

  int86x(0x10, &pc_cpu, &pc_cpu, &s_cpu); /*...ueber Interrupt 10 aktivieren*/

  pc_cpu.h.ah =0x10;               /* DAC-Bereich zuordnen (Page-Mode)      */
  pc_cpu.h.al =0x13;
  pc_cpu.h.bl =0x00;               /* Page-Mode festlegen und ...           */
  pc_cpu.h.bh =0x01;               /* 16 Bloecke mit je 16 DAC-Registern ...*/
  int86(0x10, &pc_cpu, &pc_cpu);   /* ... durch Interrupt 10 aktivieren.    */

  pc_cpu.h.ah =0x10;
  pc_cpu.h.al =0x13;
  pc_cpu.h.bl =0x01;               /* Im Page-Mode den DAC-Registerblock ...*/
  pc_cpu.h.bh =0x00;               /* ... Nummer 0 anwaehlen und ...        */
  int86(0x10, &pc_cpu, &pc_cpu);   /* ... durch Interrupt 10 aktivieren.    */

  for (i=0; i<16; i++) AC_Index[i]=i;  /* Zuordnung AC-Index zu DAC-Index   */
  AC_Index[16]=6;                  /* Letzer Eintrag ist das Overscan-Reg.  */

  pc_cpu.h.ah=0x10;                /* Aktiviere Modus zum Setzen aller ...  */
  pc_cpu.h.al=0x02;                /* ... AC-Palettenregister (LUT)         */
  s_cpu.es   =FP_SEG(AC_Index   ); /* Segmentadresse der AC-Index-Tabelle   */
  pc_cpu.x.dx=FP_OFF(AC_Index   ); /* Offsetadresse in diesem Segment und...*/
  int86x(0x10, &pc_cpu, &pc_cpu, &s_cpu); /*...ueber Interrupt 10 aktivieren*/

}

/*****************************************************************************
*                                                                            *
* void      Show_Pic(int BildNr, int Position)                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Zeigt das Bild mit der Nummer BildNr an der Stelle Position an.            *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 19.08.1992                                               *
* Letzte Aenderung: 23.10.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: BildNr:   0 <= BildNr   <  MAX_PICTURE                              *
*        Position: 1 <= Position <= 2                                        *
*                  1 = links oben                                            *
*                  2 = rechts oben                                           *
*                                                                            *
******************************************************************************
*                                                                            *
* Output:                                                                    *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: PlotPixel(), Set_LUT(), Graph_Mode(), Text_Mode()              *
*                                                                            *
*****************************************************************************/

void Show_Pic(int BildNr, int Position)
{
  unsigned char Color;             /* geklippter Farb-/Grauwertindex        */
  long s, z;                       /* Laufvariablen Spalten und Zeilen      */
  long Index, PositionOffset;

  PositionOffset = (Position==1) ? 0:POS_OFFSET;

  if ((BildNr==0) || (BildNr==1)) {
     for (z=0; z<(long)Bild[BildNr].Zeilen; z++) {
         Index = z*(long)Bild[BildNr].Spalten + (long)BildNr*(long)PIC_SIZE;
         for (s=0; s<(long)Bild[BildNr].Spalten; s++) {
             Color = Pictures[Index+s];
             Color = (Color<CLIP_COLOR) ? SCHWARZ:Color; /* Klipping durchf.*/
             PlotPixel(s+PositionOffset, z, Color);
         }
     }
     /* Bild-Platz loeschen falls neues Bild nicht die volle Groese hat     */
     for (z=(long)Bild[BildNr].Zeilen; z<Z_RES_MAX; z++)
         for (s=0; s<S_RES_MAX; s++)
             PlotPixel(s+PositionOffset, z, SCHWARZ);
     for (s=(long)Bild[BildNr].Spalten; s<S_RES_MAX; s++)
         for (z=0; z<Z_RES_MAX; z++)
             PlotPixel(s+PositionOffset, z, SCHWARZ);
  }

}
